<?php

class ControllerExtensionPaymentTcbpay extends Controller {

	public function index() {
		$this->load->language('extension/payment/tcbpay');
		$data['api_js'] = 'https://gateway2.tcbpay.com/services/proxynization_api.js';
        if ($this->config->get('payment_tcbpay_testmode')) {
            $data['api_js'] = 'https://gateway2-stg.tcbpay.com/services/proxynization_api.js';
        }
        $data['ajax_url'] = $this->url->link('extension/payment/tcbpay/confirm');
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['usecvv'] = $this->config->get('payment_tcbpay_usecvv');
        $data['expiry_months'] = [];
        if (extension_loaded('intl')) {
		    $dateFrmtm = new IntlDateFormatter($this->language->get('code'), IntlDateFormatter::FULL, IntlDateFormatter::NONE, null, null, 'LLLL');
		}

		for ($i=1; $i<13; $i++) {
			$label = date('F', mktime(0,0,0,$i,1,2000));
			if (extension_loaded('intl')) {
				$label = $dateFrmtm->format(mktime(0,0,0,$i,1,2000));
			}
			$data['expiry_months'][] = array('value' => sprintf('%02d', $i), 'label' => $label);
        }

        if (extension_loaded('intl')) {
	        $dateFrmty = new IntlDateFormatter($this->language->get('code'), IntlDateFormatter::NONE, IntlDateFormatter::NONE, null, null, 'yyyy');
	    }
		$data['expiry_years'] = [];
        $today = getdate(); 
        for ($i=$today['year']; $i < $today['year']+10; $i++) {
        	$label = date('Y', mktime(0,0,0,1,1,$i));
			if (extension_loaded('intl')) {
				$label = $dateFrmty->format(mktime(0,0,0,1,1,$i));
			}
			$data['expiry_years'][] = array('value' => substr($label, 2), 'label' => $label);
        }

        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $data['cardholder'] = $order_info['firstname'] . ' ' . $order_info['lastname'];

		return $this->load->view('extension/payment/tcbpay', $data);
	}

	public function confirm() {
		$this->load->language('extension/payment/tcbpay');
		$json = array();
		
		if ($this->session->data['order_id'] && $this->session->data['payment_method']['code'] == 'tcbpay') {
			$this->load->library('tcbpay');
			$post = $this->request->post;
			$ccNumber = '';
			$encCardNum = trim($post['enccardnum']);
			$ccExpMonth = trim($post['expiry_month']);
			$ccExpYear = trim($post['expiry_year']);
			$cardCVV = '';
			if ($this->config->get('payment_tcbpay_usecvv')) {
				$cardCVV = trim($post['cardcvv']);
			}

			if (!$post['cardholder']) {
				$json['errors']['cardholder'] = $this->language->get('require_cardholder');
			}
			if (!$encCardNum) {
				$json['errors']['cardnumber'] = $this->language->get('require_cardnumber');
			} else {
				$response = $this->tcbpay->decodeCard($encCardNum);
				$dcresp = $response['resp'];

				if ($response['success']) {
					$ccNumber = preg_replace('/[\-\s]+/', '', trim($dcresp->accountNumber));
				} elseif($dcresp) {
					$json['errors']['cardnumber'] = $dcresp->description;
				} else {
					$json['errors']['cardnumber'] = 'TCBPay gateway error.';
				}
			}
			if ($this->config->get('payment_tcbpay_usecvv') && !$cardCVV) {
				$json['errors']['cardcvv'] = $this->language->get('require_cardcvv');
			}

			if (!isset($json['errors'])) {
				$cardvalidation = $this->tcbpay->card_validation();
				$cvld = $cardvalidation->validate($ccNumber, $ccExpMonth, $ccExpYear);

		        switch (true) {
					case $cvld === -1:
						$json['error'] = sprintf($this->language->get('error_ccaccept'), substr($cardvalidation->cc_number, 0, 4));
						break;
					case $cvld === -2:
					case $cvld === -3:
					case $cvld === -4:
						$json['error'] = $this->language->get('error_expiry');
						break;
					case $cvld === -5:
					case $cvld === -6:
					case $cvld === -7:
						$json['error'] = $this->language->get('error_cvv');
						break;
					case $cvld === -8:
						$json['error'] = $this->language->get('error_ccnot_accept');
						break;
					case $cvld === false:
						$json['error'] = $this->language->get('error_ccinvalid');
						break;
		        }
				if (!in_array($cardvalidation->cc_type, $this->config->get('payment_tcbpay_cctypes'))) {
					$json['error'] = $this->language->get('error_ccnot_accept');
				}
				if ($this->config->get('payment_tcbpay_usecvv') && !preg_match($cardvalidation->getCvvRegEx()[$cardvalidation->cc_type], $post['cardcvv'])) {
					$json['error'] = $this->language->get('error_cvv');
				}
			}

			if (!$json) {
				$this->load->model('checkout/order');
				$this->load->model('localisation/country');
				$this->load->model('localisation/zone');
				$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

				$post_data = array(
		            "amount"            => (float)$order_info['total'],
		            "creditcard_number" => $encCardNum,                
		            "expire_month"      => str_pad($ccExpMonth, 2, 0, STR_PAD_LEFT),
		            "expire_year"       => $ccExpYear,
					"cvv"               => $cardCVV,
		            "first_name"        => $order_info['payment_firstname'],
		            "last_name"         => $order_info['payment_lastname'],
		            "address"           => $order_info['payment_address_1'],
		            "city"              => $order_info['payment_city'],
		            "state"             => $order_info['payment_zone_code'],
		            "zip_code"          => $order_info['payment_postcode'],
		            "country_code"      => $order_info['payment_iso_code_2'],
		            "phone_number"      => $order_info['telephone'],
		            "email"             => $order_info['email'],
		            "currency"          => $order_info['currency_code'],
		            "memo"				=> $order_info['comment']
		        );

				$response = $this->tcbpay->sendRequest($post_data);
				$resp = $response['resp'];

				if ($response['success']) {
					$message = '';
					$message .= 'Transaction Id: ' . $resp->transaction_id . "\n";
					$this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('payment_tcbpay_order_status_id'), $message, true);

					$json['redirect'] = $this->url->link('checkout/success');
				} elseif($resp) {
					$json['error'] = $resp->description;
				} else {
					$json['error'] = 'TCBPay gateway error.';
				}
			}
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
}