<?php

class Tcbpay {
    const API_LIVE_URL = 'https://gateway2.tcbpay.com/mobile_plugin/';
    const API_TEST_URL = 'https://gateway2-stg.tcbpay.com/mobile_plugin/';

    private $config;

    public function __construct($registry) {
        $this->config = $registry->get('config');
    }

    public function decodeCard($accountNumber = '') {
        $params = array(
            'accountNumber' => $accountNumber
        );

        return $this->gatewayRequest('proxynizationED.php', $params);
    }

    public function sendRequest($params = array()) {
        $rparams = array(
            'client_ip' => $_SERVER['REMOTE_ADDR'],
            'source' => 'API_OPENCART'
        );
        $rparams = array_merge($rparams, $params);

        return $this->gatewayRequest('sale.php', $rparams);
    }

    private function gatewayRequest($path='', $params = array()) {
        $rparams = array(
            'username' => $this->config->get('payment_tcbpay_username'),
            'api_key' => $this->config->get('payment_tcbpay_password'),
        );

        $rparams = array_merge($rparams, $params);

        $url = self::API_LIVE_URL . $path;
        if ($this->config->get('payment_tcbpay_testmode')) {
            $url = self::API_TEST_URL . $path;
        }

        $post_query =  http_build_query($rparams);

        $gmtime = strtotime(gmdate('Y-m-d H:i:s'));

        $headers = [
            'hash_timestamp: '. $gmtime,
            'hash: '. md5($rparams['username']."|".$rparams['api_key']."|".$this->config->get('payment_tcbpay_secretkey')."|".$gmtime)
        ];

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, true); 
        curl_setopt($ch,CURLOPT_POSTFIELDS, $post_query);
        $response = curl_exec($ch);

        curl_close ($ch);

        $result = ['success' => false, 'resp' => ''];
        $respObj = json_decode($response);

        if (isset($respObj) && !empty($respObj) && isset($respObj->response)) {
            $resp = $respObj->response;
            $result['resp'] = $resp;
            if($resp->code == '00') {
                $result['success'] = true;
            }
        }

        return $result;
    }

    public function card_validation() {
        return new TcbpayCardValidation();
    }
}

class TcbpayCardValidation {
    public $cc_type;
    public $cc_name;
    public $cc_number;
    public $cc_expiry_month;
    public $cc_expiry_year;

    public function validate($number, $expiry_m, $expiry_y) {
        $this->cc_number = preg_replace('/[^0-9]/', '', $number);

        if (preg_match('/^4[0-9]{12,18}$/', $this->cc_number)) {
            $this->cc_type = 'VI';
            $this->cc_name = 'Visa';
        } elseif (preg_match('/^(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}$/', $this->cc_number)) {
            $this->cc_type = 'MC';
            $this->cc_name = 'MasterCard';
        } elseif (preg_match('/^3(4|7)[0-9]{13}/', $this->cc_number)) {
            $this->cc_type = 'AE';
            $this->cc_name = 'American Express';
        } elseif (preg_match('/^(6011((0|9|[2-5])[0-9]{11,14}|(74|7[7-9]|8[6-9])[0-9]{10,13})|6(4[4-9][0-9]{13,16}|5[0-9]{14,17}))/', $this->cc_number)) {
            $this->cc_type = 'DI';
            $this->cc_name = 'Discover';
        } elseif (preg_match('/^35(2[8-9][0-9]{12,15}|[3-8][0-9]{13,16})$/', $this->cc_number)) {
            $this->cc_type = 'JCB';
            $this->cc_name = 'JCB';
        } elseif (preg_match('/^\\d+$/', $this->cc_number)) {
            $this->cc_type = 'OT';
            $this->cc_name = 'Other';
        } else {
            return -1;
        }

        if (is_numeric($expiry_m) && ($expiry_m > 0) && ($expiry_m < 13)) {
            $this->cc_expiry_month = $expiry_m;
        } else {
            return -2;
        }

        $current_year = date('Y');
        $expiry_y = substr($current_year, 0, 2) . $expiry_y;
        if (is_numeric($expiry_y) && ($expiry_y >= $current_year) && ($expiry_y <= ($current_year + 10))) {
            $this->cc_expiry_year = $expiry_y;
        } else {
            return -3;
        }

        if ($expiry_y == $current_year) {
            if ($expiry_m < date('n')) {
                return -4;
            }
        }

        return $this->is_valid();
    }

    public function is_valid() {
        $cardNumber = strrev($this->cc_number);
        $numSum = 0;

        for ($i=0; $i<strlen($cardNumber); $i++) {
            $currentNum = substr($cardNumber, $i, 1);

            // Double every second digit
            if ($i % 2 == 1) {
                $currentNum *= 2;
            }

            // Add digits of 2-digit numbers together
            if ($currentNum > 9) {
                $firstNum = $currentNum % 10;
                $secondNum = ($currentNum - $firstNum) / 10;
                $currentNum = $firstNum + $secondNum;
            }

            $numSum += $currentNum;
        }

        // If the total has no remainder it's OK
        return ($numSum % 10 == 0);
    }

    public function getCvvRegEx()
    {
        $verificationExpList = [
            'VI' => '/^[0-9]{3}$/',
            'MC' => '/^[0-9]{3}$/',
            'AE' => '/^[0-9]{3,4}$/',
            'DI' => '/^[0-9]{3}$/',
            'OT' => '/^[0-9]{3,4}$/',
            'JCB' => '/^[0-9]{3,4}$/'
        ];

        return $verificationExpList;
    }
}